@@ -94,7 +94,7 @@ static CamelProvider exchange_provider = {
 	CAMEL_PROVIDER_IS_REMOTE | CAMEL_PROVIDER_IS_SOURCE |
 	CAMEL_PROVIDER_IS_STORAGE | CAMEL_PROVIDER_IS_EXTERNAL,
 
-	CAMEL_URL_NEED_USER | CAMEL_URL_ALLOW_AUTH | CAMEL_URL_HIDDEN_HOST,
+	CAMEL_URL_NEED_USER | CAMEL_URL_HIDDEN_AUTH | CAMEL_URL_HIDDEN_HOST,
 
 	exchange_conf_entries 
 
@@ -133,6 +133,8 @@ exchange_auto_detect_cb (CamelURL *url, GHashTable **auto_detected,
 			     g_strdup (url->user));
 	g_hash_table_insert (*auto_detected, g_strdup ("pf_server"),
 			     g_strdup (url->host));
+	g_hash_table_insert (*auto_detected, g_strdup ("ad_server"),
+			     g_strdup (camel_url_get_param (url, "ad_server")));
 
 	return 0;
 }
@@ -143,12 +145,13 @@ exchange_validate_user_cb (CamelURL *camel_url, const char *owa_url,
 {
 	gboolean valid;
 	char *host = NULL;
+	char *ad_server = NULL;
 
 	valid = e2k_validate_user (owa_url, camel_url->user, 
-				   &host, remember_password);
+				   &host, &ad_server, remember_password);
 
-	camel_url_set_host(camel_url, valid?host:NULL);
-	g_free(host);
+	camel_url_set_host(camel_url, valid?host:"");
+	camel_url_set_param(camel_url, "ad_server", valid?ad_server:NULL);
 
 	return valid;
 }
