@@ -25,9 +25,9 @@
 #include "exchange-account.h"
 #include "exchange-hierarchy.h"
 #include "e2k-uri.h"
+#include "e2k-path.h"
 #include "exchange-config-listener.h"
 
-#include <e-util/e-path.h>
 #include <gal/util/e-util.h>
 #include <gal/util/e-xml-utils.h>
 #include <libxml/xmlmemory.h>
@@ -122,9 +122,6 @@ e_folder_exchange_new (ExchangeHierarchy *hier, const char *name,
 {
 	EFolderExchange *efe;
 	EFolder *ef;
-	char *conf_key_cal="/apps/evolution/calendar/sources";
-	char *conf_key_tasks="/apps/evolution/tasks/sources";
-	char *conf_key_contacts="/apps/evolution/addressbook/sources";
 	ESourceList *cal_source_list, *task_source_list, *cont_source_list;
 
 	g_return_val_if_fail (EXCHANGE_IS_HIERARCHY (hier), NULL);
@@ -153,13 +150,12 @@ e_folder_exchange_new (ExchangeHierarchy *hier, const char *name,
 		    (strcmp (type, "calendar/public") == 0)) {
 			cal_source_list = e_source_list_new_for_gconf ( 
 						gconf_client_get_default (), 
-						conf_key_cal);
+						CONF_KEY_CAL);
 			add_esource (hier->account, 
-				     conf_key_cal, 
+				     EXCHANGE_CALENDAR_FOLDER, 
 				     name, 
 				     physical_uri, 
-				     &cal_source_list,
-				     FALSE);
+				     &cal_source_list);
 			e_source_list_sync (cal_source_list, NULL);
 			g_object_unref (cal_source_list);
 		}
@@ -167,13 +163,12 @@ e_folder_exchange_new (ExchangeHierarchy *hier, const char *name,
 			 (strcmp (type, "tasks/public") == 0)) {
 			task_source_list = e_source_list_new_for_gconf ( 
 						gconf_client_get_default (), 
-						conf_key_tasks);
+						CONF_KEY_TASKS);
 			add_esource (hier->account, 
-				     conf_key_tasks, 
+				     EXCHANGE_TASKS_FOLDER, 
 				     name, 
 				     physical_uri, 
-				     &task_source_list,
-				     FALSE);
+				     &task_source_list);
 			e_source_list_sync (task_source_list, NULL);
 			g_object_unref (task_source_list);
 		}
@@ -181,13 +176,12 @@ e_folder_exchange_new (ExchangeHierarchy *hier, const char *name,
 			 (strcmp (type, "contacts/public") == 0)) {
 			cont_source_list = e_source_list_new_for_gconf ( 
 						gconf_client_get_default (), 
-						conf_key_contacts);
+						CONF_KEY_CONTACTS);
 			add_esource (hier->account, 
-				     conf_key_contacts, 
+				     EXCHANGE_CONTACTS_FOLDER, 
 				     name, 
 				     physical_uri, 
-				     &cont_source_list,
-				     TRUE);
+				     &cont_source_list);
 			e_source_list_sync (cont_source_list, NULL);
 			g_object_unref (cont_source_list);
 		}
@@ -299,7 +293,7 @@ e_folder_exchange_set_permanent_uri (EFolder *folder, const char *permanent_uri)
 long long int
 e_folder_exchange_get_folder_size (EFolder *folder)
 {
-	g_return_val_if_fail (E_IS_FOLDER_EXCHANGE (folder), 0);
+	g_return_val_if_fail (E_IS_FOLDER_EXCHANGE (folder), -1);
 
 	return E_FOLDER_EXCHANGE (folder)->priv->folder_size;
 }
@@ -431,6 +425,7 @@ e_folder_exchange_save_to_file (EFolder *folder, const char *filename)
 	const char *name, *type, *outlook_class;
 	const char *physical_uri, *internal_uri, *permanent_uri;
 	const char *folder_size;
+	long long int fsize;
 	int status;
 
 	name = e_folder_get_name (folder);
@@ -439,7 +434,10 @@ e_folder_exchange_save_to_file (EFolder *folder, const char *filename)
 	physical_uri = e_folder_get_physical_uri (folder);
 	internal_uri = e_folder_exchange_get_internal_uri (folder);
 	permanent_uri = e_folder_exchange_get_permanent_uri (folder);
-	folder_size = g_strdup_printf ("%llu", e_folder_exchange_get_folder_size (folder));
+	if ((fsize = e_folder_exchange_get_folder_size (folder)) >= 0)
+		folder_size = g_strdup_printf ("%llu", fsize);
+	else
+		return FALSE;
 
 	g_return_val_if_fail (name && type && physical_uri && internal_uri,
 			      FALSE);
@@ -583,6 +581,8 @@ e_folder_exchange_propfind (EFolder *folder, E2kOperation *op,
 			    const char **props, int nprops,
 			    E2kResult **results, int *nresults)
 {
+	g_return_val_if_fail (E_IS_FOLDER_EXCHANGE (folder), E2K_HTTP_MALFORMED);
+
 	return e2k_context_propfind (
 		E_FOLDER_EXCHANGE_CONTEXT (folder), op,
 		E_FOLDER_EXCHANGE_URI (folder),
@@ -609,6 +609,8 @@ e_folder_exchange_bpropfind_start (EFolder *folder, E2kOperation *op,
 				   const char **hrefs, int nhrefs,
 				   const char **props, int nprops)
 {
+	g_return_val_if_fail (E_IS_FOLDER_EXCHANGE (folder), NULL);
+
 	return e2k_context_bpropfind_start (
 		E_FOLDER_EXCHANGE_CONTEXT (folder), op,
 		E_FOLDER_EXCHANGE_URI (folder),
@@ -636,6 +638,8 @@ e_folder_exchange_search_start (EFolder *folder, E2kOperation *op,
 				E2kRestriction *rn, const char *orderby,
 				gboolean ascending)
 {
+	g_return_val_if_fail (E_IS_FOLDER_EXCHANGE (folder), NULL);
+
 	return e2k_context_search_start (
 		E_FOLDER_EXCHANGE_CONTEXT (folder), op,
 		E_FOLDER_EXCHANGE_URI (folder),
@@ -662,6 +666,8 @@ e_folder_exchange_subscribe (EFolder *folder,
 			     E2kContextChangeCallback callback,
 			     gpointer user_data)
 {
+	g_return_if_fail (E_IS_FOLDER_EXCHANGE (folder));
+
 	e2k_context_subscribe (E_FOLDER_EXCHANGE_CONTEXT (folder),
 			       E_FOLDER_EXCHANGE_URI (folder),
 			       type, min_interval, callback, user_data);
@@ -677,8 +683,15 @@ e_folder_exchange_subscribe (EFolder *folder,
 void
 e_folder_exchange_unsubscribe (EFolder *folder)
 {
-	e2k_context_unsubscribe (E_FOLDER_EXCHANGE_CONTEXT (folder),
-				 E_FOLDER_EXCHANGE_URI (folder));
+	g_return_if_fail (E_IS_FOLDER_EXCHANGE (folder));
+
+	/* FIXME : This is a hack as of now. The free_folder in mail-stub
+	gets called when we are in offline and the context is NULL then. */
+	E2kContext *ctx = E_FOLDER_EXCHANGE_CONTEXT (folder);
+	if (ctx) {
+		e2k_context_unsubscribe (E_FOLDER_EXCHANGE_CONTEXT (folder),
+					 E_FOLDER_EXCHANGE_URI (folder));
+	}
 }
 
 /**
@@ -700,6 +713,8 @@ e_folder_exchange_transfer_start (EFolder *source, E2kOperation *op,
 				  EFolder *dest, GPtrArray *source_hrefs,
 				  gboolean delete_originals)
 {
+	g_return_val_if_fail (E_IS_FOLDER_EXCHANGE (source), NULL);
+
 	return e2k_context_transfer_start (E_FOLDER_EXCHANGE_CONTEXT (source), op,
 					   E_FOLDER_EXCHANGE_URI (source),
 					   E_FOLDER_EXCHANGE_URI (dest),
@@ -736,6 +751,8 @@ e_folder_exchange_put_new (EFolder *folder,
 			   const char *body, int length,
 			   char **location, char **repl_uid)
 {
+	g_return_val_if_fail (E_IS_FOLDER_EXCHANGE (folder), E2K_HTTP_MALFORMED);
+
 	return e2k_context_put_new (E_FOLDER_EXCHANGE_CONTEXT (folder), op,
 				    E_FOLDER_EXCHANGE_URI (folder),
 				    object_name, test_callback, user_data,
@@ -770,6 +787,8 @@ e_folder_exchange_proppatch_new (EFolder *folder, E2kOperation *op,
 				 E2kProperties *props,
 				 char **location, char **repl_uid)
 {
+	g_return_val_if_fail (E_IS_FOLDER_EXCHANGE (folder), E2K_HTTP_MALFORMED);
+
 	return e2k_context_proppatch_new (E_FOLDER_EXCHANGE_CONTEXT (folder), op,
 					  E_FOLDER_EXCHANGE_URI (folder),
 					  object_name,
@@ -797,6 +816,8 @@ e_folder_exchange_bproppatch_start (EFolder *folder, E2kOperation *op,
 				    const char **hrefs, int nhrefs,
 				    E2kProperties *props, gboolean create)
 {
+	g_return_val_if_fail (E_IS_FOLDER_EXCHANGE (folder), NULL);
+
 	return e2k_context_bproppatch_start (E_FOLDER_EXCHANGE_CONTEXT (folder), op,
 					     E_FOLDER_EXCHANGE_URI (folder),
 					     hrefs, nhrefs, props, create);
@@ -819,6 +840,8 @@ E2kResultIter *
 e_folder_exchange_bdelete_start (EFolder *folder, E2kOperation *op,
 				 const char **hrefs, int nhrefs)
 {
+	g_return_val_if_fail (E_IS_FOLDER_EXCHANGE (folder), NULL);
+
 	return e2k_context_bdelete_start (E_FOLDER_EXCHANGE_CONTEXT (folder), op,
 					  E_FOLDER_EXCHANGE_URI (folder),
 					  hrefs, nhrefs);
@@ -843,6 +866,8 @@ e_folder_exchange_mkcol (EFolder *folder, E2kOperation *op,
 			 E2kProperties *props,
 			 char **permanent_url)
 {
+	g_return_val_if_fail (E_IS_FOLDER_EXCHANGE (folder), E2K_HTTP_MALFORMED);
+
 	return e2k_context_mkcol (E_FOLDER_EXCHANGE_CONTEXT (folder), op,
 				  E_FOLDER_EXCHANGE_URI (folder),
 				  props, permanent_url);
@@ -862,9 +887,6 @@ E2kHTTPStatus
 e_folder_exchange_delete (EFolder *folder, E2kOperation *op)
 {
 	ExchangeHierarchy *hier;
-	char *conf_key_cal="/apps/evolution/calendar/sources";
-	char *conf_key_tasks="/apps/evolution/tasks/sources";
-	char *conf_key_contacts="/apps/evolution/addressbook/sources";
 	ESourceList *cal_source_list, *task_source_list, *cont_source_list;
 	const char *folder_type, *physical_uri;
 
@@ -882,12 +904,12 @@ e_folder_exchange_delete (EFolder *folder, E2kOperation *op)
 		    (strcmp (folder_type, "calendar/public") == 0)) {
 			cal_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						conf_key_cal);
+						CONF_KEY_CAL);
 			remove_esource (hier->account, 
-					conf_key_cal,
+					EXCHANGE_CALENDAR_FOLDER,
 					physical_uri,
 					&cal_source_list,
-					FALSE, FALSE);
+					FALSE);
 			e_source_list_sync (cal_source_list, NULL);
 			g_object_unref (cal_source_list);
 		}
@@ -895,12 +917,12 @@ e_folder_exchange_delete (EFolder *folder, E2kOperation *op)
 			 (strcmp (folder_type, "tasks/public") == 0)) {
 			task_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						conf_key_tasks);
+						CONF_KEY_TASKS);
 			remove_esource (hier->account,
-					conf_key_tasks,
+					EXCHANGE_TASKS_FOLDER,
 					physical_uri,
 					&task_source_list,
-					FALSE, FALSE);
+					FALSE);
 			e_source_list_sync (task_source_list, NULL);
 			g_object_unref (task_source_list);
 		}
@@ -908,12 +930,12 @@ e_folder_exchange_delete (EFolder *folder, E2kOperation *op)
 			 (strcmp (folder_type, "contacts/public") == 0)) { 
 			cont_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						conf_key_contacts);
+						CONF_KEY_CONTACTS);
 			remove_esource (hier->account,
-					conf_key_contacts,
+					EXCHANGE_CONTACTS_FOLDER,
 					physical_uri,
 					&cont_source_list,
-					FALSE, TRUE);
+					FALSE);
 			e_source_list_sync (cont_source_list, NULL);
 			g_object_unref (cont_source_list);
 		}
@@ -943,6 +965,8 @@ e_folder_exchange_transfer_dir (EFolder *source, E2kOperation *op,
 				EFolder *dest, gboolean delete_original,
 				char **permanent_url)
 {
+	g_return_val_if_fail (E_IS_FOLDER_EXCHANGE (source), E2K_HTTP_MALFORMED);
+
 	return e2k_context_transfer_dir (E_FOLDER_EXCHANGE_CONTEXT (source), op,
 					 E_FOLDER_EXCHANGE_URI (source),
 					 E_FOLDER_EXCHANGE_URI (dest),
