@@ -32,7 +32,6 @@
 #include "exchange-hierarchy-favorites.h"
 #include "exchange-hierarchy-foreign.h"
 #include "exchange-hierarchy-gal.h"
-#include "exchange-offline-listener.h"
 #include "e-folder-exchange.h"
 #include "e2k-autoconfig.h"
 #include "e2k-encoding-utils.h"
@@ -42,7 +41,7 @@
 #include "e2k-uri.h"
 #include "e2k-utils.h"
 #include <e-util/e-dialog-utils.h>
-#include <e-util/e-passwords.h>
+#include <libedataserverui/e-passwords.h>
 
 #include <gal/util/e-util.h>
 #include <libgnome/gnome-util.h>
@@ -76,10 +75,10 @@ struct _ExchangeAccountPrivate {
 	ExchangeHierarchy *favorites_hierarchy;
 	GHashTable *folders;
 	char *uri_authority, *http_uri_schema;
-	gboolean uris_use_email;
+	gboolean uris_use_email, offline_sync;
 
 	char *identity_name, *identity_email, *source_uri, *password_key;
-	char *username, *password, *windows_domain, *ad_server, *offline_sync;
+	char *username, *password, *windows_domain, *ad_server;
 	char *owa_url;
 	E2kAutoconfigAuthPref auth_pref;
 	int ad_limit, passwd_exp_warn_period;
@@ -286,9 +285,6 @@ finalize (GObject *object)
 	if (account->priv->ad_server)
 		g_free (account->priv->ad_server);
 
-	if (account->priv->offline_sync)
-		g_free (account->priv->offline_sync);
-
 	if (account->priv->owa_url)
 		g_free (account->priv->owa_url);
 
@@ -713,7 +709,7 @@ exchange_account_open_folder (ExchangeAccount *account, const char *path)
 {
 	ExchangeHierarchy *hier;
 	EFolder *folder;
-	gboolean offline = exchange_account_is_offline (account);
+	int offline;
 
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), 
 				EXCHANGE_ACCOUNT_FOLDER_GENERIC_ERROR);
@@ -721,7 +717,8 @@ exchange_account_open_folder (ExchangeAccount *account, const char *path)
 	if (!get_folder (account, path, &folder, &hier))
 		return EXCHANGE_ACCOUNT_FOLDER_DOES_NOT_EXIST;
 
-	if (!offline && !account->priv->connected &&
+	exchange_account_is_offline (account, &offline);
+	if (offline == ONLINE_MODE && !account->priv->connected &&
 	    hier == (ExchangeHierarchy *)account->priv->hierarchies->pdata[0] &&
 	    folder == hier->toplevel) {
 		/* The shell is asking us to open the personal folders
@@ -730,8 +727,8 @@ exchange_account_open_folder (ExchangeAccount *account, const char *path)
 		 */
 		return EXCHANGE_ACCOUNT_FOLDER_DOES_NOT_EXIST;
 	}		
-
-	return exchange_hierarchy_scan_subtree (hier, folder, offline);
+	
+	return exchange_hierarchy_scan_subtree (hier, folder, (offline == OFFLINE_MODE));
 }
 
 ExchangeAccountFolderResult
@@ -849,28 +846,34 @@ get_password (ExchangeAccount *account, E2kAutoconfig *ac, const char *errmsg)
 
 	password = e_passwords_get_password ("Exchange", account->priv->password_key);
 
-	if (!password && exchange_component_is_interactive (global_exchange_component)) {
-		char *prompt;
+	if (exchange_component_is_interactive (global_exchange_component)) {
+		if (!password) {
+			char *prompt;
 
-		prompt = g_strdup_printf (_("%sEnter password for %s"),
-					  errmsg, account->account_name);
-		oldremember = remember = account->priv->account->source->save_passwd;
-		password = e_passwords_ask_password (
-			_("Enter password"),
-			"Exchange", account->priv->password_key,
-			prompt, E_PASSWORDS_REMEMBER_FOREVER|E_PASSWORDS_SECRET,
-			&remember, NULL);
-		if (remember != oldremember) {
-			account->priv->account->source->save_passwd = remember;
+			prompt = g_strdup_printf (_("%sEnter password for %s"),
+						  errmsg, account->account_name);
+			oldremember = remember = 
+					account->priv->account->source->save_passwd;
+			password = e_passwords_ask_password (
+					_("Enter password"),
+					"Exchange", 
+					account->priv->password_key,
+					prompt, 
+					E_PASSWORDS_REMEMBER_FOREVER|E_PASSWORDS_SECRET,
+					&remember, 
+					NULL);
+			if (remember != oldremember) {
+				account->priv->account->source->save_passwd = remember;
+			}
+			g_free (prompt);
+		} 
+		else if (!account->priv->account->source->save_passwd) {
+			/* get_password returns the password cached but user has not 
+		 	 * selected remember password option, forget this password 
+		 	 * whis is stored temporarily by e2k_validate_user() 
+		 	 */
+			e_passwords_forget_password ("Exchange", account->priv->password_key);
 		}
-		g_free (prompt);
-	} 
-	else if (password && !account->priv->account->source->save_passwd) {
-		/* get_password returns the password cached but user has not 
-		 * selected remember password option, forget this password 
-		 * whis is stored temporarily by e2k_validate_user() 
-		 */
-		e_passwords_forget_password ("Exchange", account->priv->password_key);
 	}
 
 	if (password) {
@@ -910,14 +913,20 @@ is_password_expired (ExchangeAccount *account, E2kAutoconfig *ac)
 }
 
 static void
-change_passwd_cb (GtkWidget *button, char *old_passwd)
+change_passwd_cb (GtkWidget *button, ExchangeAccount *account)
 {
+	char *current_passwd, *new_passwd;
+
 	gtk_widget_hide (gtk_widget_get_toplevel(button));
-	exchange_get_new_password (old_passwd, TRUE);
+	current_passwd = exchange_account_get_password (account);
+	new_passwd = exchange_get_new_password (current_passwd, TRUE);
+	exchange_account_set_password (account, current_passwd, new_passwd);
+	g_free (current_passwd);
+	g_free (new_passwd);
 }
 
 static void
-display_passwd_expiry_message (int max_passwd_age, char *old_passwd)
+display_passwd_expiry_message (int max_passwd_age, ExchangeAccount *account)
 {
 	GladeXML *xml;
 	GtkWidget *top_widget, *change_passwd_button;
@@ -942,7 +951,7 @@ display_passwd_expiry_message (int max_passwd_age, char *old_passwd)
 	g_signal_connect (change_passwd_button, 
 			  "clicked", 
 			  G_CALLBACK (change_passwd_cb), 
-			  old_passwd);
+			  account);
 
 	response = gtk_dialog_run (GTK_DIALOG (top_widget));
 
@@ -958,7 +967,10 @@ find_passwd_exp_period (ExchangeAccount *account, E2kGlobalCatalogEntry *entry)
 	int max_pwd_age_days;
 	E2kOperation gcop;
 	E2kGlobalCatalogStatus gcstatus;
-	char *current_passwd;
+
+	/* If user has not selected password expiry warning option, return */
+	if (account->priv->passwd_exp_warn_period == -1)
+		return;
 
 	/* Check for password expiry period */ 
 	/* This needs to be invoked after is_password_expired(), i.e., 
@@ -993,10 +1005,8 @@ find_passwd_exp_period (ExchangeAccount *account, E2kGlobalCatalogEntry *entry)
 		( max_pwd_age * ONE_HUNDRED_NANOSECOND ) / SECONDS_IN_DAY;
 
 		if (max_pwd_age_days <= account->priv->passwd_exp_warn_period) {
-			/* This we need for changing password */
-			current_passwd = exchange_account_get_password (account);
 			display_passwd_expiry_message (max_pwd_age_days, 
-						       current_passwd);
+						       account);
 		}
 	} 
 }
@@ -1095,15 +1105,18 @@ exchange_account_set_offline (ExchangeAccount *account)
 gboolean
 exchange_account_set_online (ExchangeAccount *account)
 {
-		
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), FALSE);
 
-	account->priv->account_online = TRUE;
+	if (!account->priv->account_online) {
+		account->priv->account_online = TRUE;
 
-	if (exchange_account_connect (account))
+		if (exchange_account_connect (account))
+			return TRUE;
+		else
+			return FALSE;
+	} else {
 		return TRUE;
-	else
-		return FALSE;
+	}
 }
 
 /**
@@ -1112,10 +1125,14 @@ exchange_account_set_online (ExchangeAccount *account)
  *
  * Return value: Returns TRUE if account is offline
  **/
-gboolean
-exchange_account_is_offline (ExchangeAccount *account)
+void
+exchange_account_is_offline (ExchangeAccount *account, int *state)
 {
-	return exchange_component_is_offline (global_exchange_component);
+	*state = UNSUPPORTED_MODE;
+
+	g_return_if_fail (EXCHANGE_IS_ACCOUNT (account));
+
+	exchange_component_is_offline (global_exchange_component, state);
 }
 
 
@@ -1127,7 +1144,12 @@ setup_account_hierarchies (ExchangeAccount *account)
 	char *phys_uri_prefix, *dir;
 	DIR *d;
 	struct dirent *dent;
-	gboolean offline = exchange_account_is_offline (account);
+	int offline;
+
+	exchange_account_is_offline (account, &offline);
+
+	if (offline == UNSUPPORTED_MODE)
+		return FALSE;
 
 	/* Set up Personal Folders hierarchy */
 	phys_uri_prefix = g_strdup_printf ("exchange://%s/personal",
@@ -1209,7 +1231,7 @@ setup_account_hierarchies (ExchangeAccount *account)
 	 */
 	fresult = exchange_hierarchy_scan_subtree (personal_hier,
 						   personal_hier->toplevel,
-						   offline);
+						   (offline == OFFLINE_MODE));
 	if (fresult != EXCHANGE_ACCOUNT_FOLDER_OK) {
 		account->priv->connecting = FALSE;
 		return FALSE;
@@ -1218,7 +1240,7 @@ setup_account_hierarchies (ExchangeAccount *account)
 	fresult = exchange_hierarchy_scan_subtree (
 		account->priv->favorites_hierarchy,
 		account->priv->favorites_hierarchy->toplevel,
-		offline);
+		(offline == OFFLINE_MODE));
 	if (fresult != EXCHANGE_ACCOUNT_FOLDER_OK) {
 		account->priv->connecting = FALSE;
 		return FALSE;
@@ -1256,15 +1278,16 @@ exchange_account_connect (ExchangeAccount *account)
 	E2kOperation gcop;
 	const char *quota_msg = NULL;
 	char *user_name = NULL;
-	int offline = exchange_component_is_offline (global_exchange_component);
+	int offline;
 
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), NULL);
 
+	exchange_account_is_offline (account, &offline);
 	g_mutex_lock (account->priv->connect_lock);
-	//if ((account->priv->connecting) || (!account->priv->account_online)){
-	if ((account->priv->connecting) || offline){
+
+	if ((account->priv->connecting) || (offline == OFFLINE_MODE)){
 		g_mutex_unlock (account->priv->connect_lock);
-		if (offline) {
+		if (offline == OFFLINE_MODE) {
 			setup_account_hierarchies (account);
 			e_notice (NULL, GTK_MESSAGE_ERROR, _("Exchange Account is offline. Cannot display folders\n"));
 		}
@@ -1447,6 +1470,9 @@ exchange_account_connect (ExchangeAccount *account)
 			account->default_timezone = g_strdup (timezone);
 	}
 
+	if (!setup_account_hierarchies (account)) 
+		return NULL;
+	
 	/* Find the password expiery peripod and display warning */
 	find_passwd_exp_period(account, entry);
 	
@@ -1473,9 +1499,6 @@ exchange_account_connect (ExchangeAccount *account)
 			e_notice (NULL, GTK_MESSAGE_INFO, quota_msg);
 	}
 	
-	if (!setup_account_hierarchies (account)) 
-		return NULL;
-	
 	account->priv->connected = TRUE;
 	account->priv->account_online = TRUE;
 	account->priv->connecting = FALSE;
@@ -1487,6 +1510,25 @@ exchange_account_connect (ExchangeAccount *account)
 	return account->priv->ctx;
 }
 
+/**
+ * exchange_account_is_offline_sync_set:
+ * @account: an #ExchangeAccount
+ *
+ * Return value: TRUE if offline_sync is set for @account and FALSE if not.
+ */
+void
+exchange_account_is_offline_sync_set (ExchangeAccount *account, int *mode)
+{
+	*mode = UNSUPPORTED_MODE;
+
+	g_return_if_fail (EXCHANGE_IS_ACCOUNT (account));
+
+	if (account->priv->offline_sync)
+		*mode = OFFLINE_MODE;
+	else
+		*mode = ONLINE_MODE;
+}
+
 /**
  * exchange_account_get_context:
  * @account: an #ExchangeAccount
@@ -1773,15 +1815,15 @@ exchange_account_new (EAccountList *account_list, EAccount *adata)
 	
 	passwd_exp_warn_period = e2k_uri_get_param (uri, "passwd_exp_warn_period");
 	if (!passwd_exp_warn_period || !*passwd_exp_warn_period)
-		account->priv->passwd_exp_warn_period = PASSWD_EXPIRY_NOTIFICATION_PERIOD;
+		account->priv->passwd_exp_warn_period = -1;
 	else
 		account->priv->passwd_exp_warn_period = atoi (passwd_exp_warn_period);
 
 	offline_sync = e2k_uri_get_param (uri, "offline_sync");
-	if (!offline_sync || !*offline_sync)
-		account->priv->offline_sync = g_strdup ("0");
-	else
-		account->priv->offline_sync = g_strdup (offline_sync);
+	if (!offline_sync) 
+		account->priv->offline_sync = FALSE;
+	else 
+		account->priv->offline_sync = TRUE;
 
 	owa_path = e2k_uri_get_param (uri, "owa_path");
 	if (!owa_path || !*owa_path)
@@ -1793,6 +1835,8 @@ exchange_account_new (EAccountList *account_list, EAccount *adata)
 	if (!pf_server || !*pf_server)
 		pf_server = uri->host;
 
+	/* Now we can set protocol reading owa_url, instead of having 
+	   use_ssl parameter */
 	proto = e2k_uri_get_param (uri, "use_ssl") ? "https" : "http";
 
 	owa_url = e2k_uri_get_param (uri, "owa_url");
